// Copyright (c) 2006, TopCoder, Inc. All rights reserved.
using System;
using System.Collections.Generic;
using System.Xml;

namespace TopCoder.Uml.Xmi.MetaData
{
    /// <summary>
    /// Represents an attribute of a UmlClass.
    /// </summary>
    /// <author>Dan Manges (Seiz3r)</author>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class UmlAttribute : XmiElement
    {
        /// <summary>
        /// Getter implementation to include in code generation output.
        /// </summary>
        private const string GET_STRING = "get\r\n            {\r\n                throw new NotImplementedException();\r\n            }\r\n";

        /// <summary>
        /// Setter implementation to include in code generation output.
        /// </summary>
        private const string SET_STRING = "set\r\n            {\r\n                throw new NotImplementedException();\r\n            }\r\n";
        
        /// <summary>
        /// Data about this attribute.
        /// </summary>
        private string name, visibility, ownerScope, type;

        /// <summary>
        /// The uml type and id reference this for this attribute type.
        /// </summary>
        private TypeRef typeRef;

        /// <summary>
        /// The namespace for the type of the attribute.
        /// </summary>
        private string @namespace;

        /// <summary>
        /// Create a new UmlAttribute from an XMI node.
        /// </summary>
        /// <param name="node">The node to read data from.</param>
        public UmlAttribute(XmlNode node)
            : base(node)
        {
            this.name = Helper.ReadAttribute(node, "name");
            this.ownerScope = Helper.ReadAttribute(node, "ownerScope");
            this.visibility = Helper.ReadAttribute(node, "visibility");
            this.ReadTypeRef(node);
        }

        /// <summary>
        /// Gets the implementation for the attribute depending if it is a getter, setter, private attribute.
        /// </summary>
        /// <returns>The implementation for the attribute.</returns>
        private string GetImplementation()
        {
            if (Array.IndexOf<string>(this.Stereotypes, "property") > -1)
            {
                return "\r\n        {\r\n            " + GET_STRING + "            " + SET_STRING + "        }";

            }
            else if (Array.IndexOf<string>(this.Stereotypes, "property-get") > -1)
            {
                return "\r\n        {\r\n            " + GET_STRING + "        }";
            }
            else if (Array.IndexOf<string>(this.Stereotypes, "property-set") > -1)
            {
                return "\r\n        {\r\n            " + SET_STRING + "        }";
            }
            else
            {
                return ";";
            }
        }

        /// <summary>
        /// Gets the attribute name.
        /// </summary>
        public string Name
        {
            get
            {
                return this.name;
            }
        }

        /// <summary>
        /// Gets or sets the namespace for the return type of the attribute.
        /// </summary>
        public string Namespace
        {
            get
            {
                return this.@namespace;
            }
            set
            {
                this.@namespace = value;
            }
        }

        /// <summary>
        /// Gets any attribute modifiers read from stereotypes in the XMI file.
        /// </summary>
        /// <remarks>Currently supports readonly, virtual, and override.</remarks>
        /// <returns>Attribute modifier string.</returns>
        private string GetModifiers()
        {
            string[] possibleTypes = new string[] { "readonly", "virtual", "override"};
            foreach (string possibleType in possibleTypes)
            {
                if (Array.IndexOf<string>(this.Stereotypes, possibleType) > -1)
                {
                    return possibleType + " ";
                }
            }
            return " ";
        }

        /// <summary>
        /// Write the attribute data to an XmlWriter.
        /// </summary>
        /// <param name="writer">The XmlWriter to use for output.</param>
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement("attribute");
            writer.WriteStartElement("documentation");
            writer.WriteCData(Helper.GetDocumentation(this, 12));
            writer.WriteEndElement();
            writer.WriteElementString("visibility", this.visibility);
            writer.WriteElementString("ownerScope", Helper.OwnerScopeToCSharp(this.ownerScope));
            writer.WriteElementString("modifiers", this.GetModifiers());
            writer.WriteElementString("type", this.Type);
            writer.WriteElementString("name", this.name);
            writer.WriteElementString("implementation", this.GetImplementation());
            writer.WriteEndElement();
        }

        /// <summary>
        /// Gets the referenced uml type of this attribute.
        /// </summary>
        public TypeRef TypeRef
        {
            get
            {
                return this.typeRef;
            }
        }

        /// <summary>
        /// Gets the type of this attribute.
        /// </summary>
        public string Type
        {
            get
            {
                return this.type;
            }
            set
            {
                this.type = value;
            }
        }

        /// <summary>
        /// Read the referenced type from the attribute node.
        /// </summary>
        /// <param name="node">The node to read from.</param>
        private void ReadTypeRef(XmlNode node)
        {
            foreach (XmlNode childNode in node.ChildNodes)
            {
                if ((childNode.Name == "UML:StructuralFeature.type" || childNode.Name == "UML2:TypedElement.type") &&
                    childNode.ChildNodes.Count == 1)
                {
                    XmlNode typeRefNode = childNode.ChildNodes[0];
                    this.typeRef = new TypeRef(Helper.StringToUmlType(typeRefNode.Name), Helper.ReadAttribute(typeRefNode, "xmi.idref", ""));
                }
            }
        }
    }
}