// Copyright (c) 2006, TopCoder, Inc. All rights reserved.
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace TopCoder.Uml.Xmi.MetaData
{
    /// <summary>
    /// Represents a UML:Class, storing attributes along with methods and properties.  Also stores necessary
    /// include namespaces.
    /// </summary>
    /// <author>Dan Manges (Seiz3r)</author>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class UmlClass : XmiElement
    {
        /// <summary>
        /// Is this an abstract class?
        /// </summary>
        private bool isAbstract;

        /// <summary>
        /// Is this an interface?
        /// </summary>
        private bool isInterface;

        /// <summary>
        /// Data about the class.
        /// </summary>
        private string visibility, @namespace, name;

        /// <summary>
        /// List of method id references for this class.
        /// </summary>
        private List<string> methodReferences = new List<string>();

        /// <summary>
        /// List of Method objects in this class.
        /// </summary>
        private List<Method> methods = new List<Method>();

        /// <summary>
        /// List of attribute id references for this class.
        /// </summary>
        private List<string> attributeReferences = new List<string>();

        /// <summary>
        /// List of UML:Attributes in this class.
        /// </summary>
        private List<UmlAttribute> umlAttributes = new List<UmlAttribute>();

        /// <summary>
        /// List of namespaces to include for this class.
        /// </summary>
        private List<string> includes = new List<string>();

        /// <summary>
        /// Create a new UmlClass, defaulting to not being an interface.
        /// </summary>
        /// <param name="node">The node to read data from.</param>
        public UmlClass(XmlNode node) : this(node, false)
        {
            // Pass the node to the other constructor.
        }

        /// <summary>
        /// Create a new UmlClass, with an option to indicate an interface.
        /// </summary>
        /// <param name="node">The node to read data from.</param>
        /// <param name="isInterface">Is this an interface?</param>
        public UmlClass(XmlNode node, bool isInterface)
            : base(node)
        {
            this.includes.Add("System");
            this.isInterface = isInterface;
            this.name = Helper.ReadAttribute(node, "name");
            this.visibility = Helper.ReadAttribute(node, "visibility");
            this.isAbstract = Helper.ReadAttribute(node, "isAbstract") == "true" ? true : false;
        }

        /// <summary>
        /// Gets whether this is an interface.
        /// </summary>
        public bool IsInterface
        {
            get
            {
                return this.isInterface;
            }
        }

        /// <summary>
        /// Add a UmlAttribute to this class.
        /// </summary>
        /// <param name="attribute">The UmlAttribute to add.</param>
        public void AddUmlAttribute(UmlAttribute attribute)
        {
            this.umlAttributes.Add(attribute);
            if (!this.includes.Contains(attribute.Namespace) && this.Namespace != attribute.Namespace)
            {
                this.includes.Add(attribute.Namespace);
            }
        }

        /// <summary>
        /// Add a Method to this class.
        /// </summary>
        /// <param name="method">The method to add.</param>
        public void AddMethod(Method method)
        {
            this.methods.Add(method);
            method.UmlClass = this;
            foreach (UmlParameter p in method.Parameters)
            {
                if (!this.includes.Contains(p.Namespace) && this.Namespace != p.Namespace)
                {
                    this.includes.Add(p.Namespace);
                }
            }
        }

        /// <summary>
        /// Gets an array of namespaces to include in this class.
        /// </summary>
        public string[] Includes
        {
            get
            {
                this.includes.Sort();
                return this.includes.ToArray();
            }
        }

        /// <summary>
        /// Gets an array of method in this class.
        /// </summary>
        public Method[] Methods
        {
            get
            {
                return this.methods.ToArray();
            }
        }

        /// <summary>
        /// Gets an array of attributes in this class.
        /// </summary>
        public UmlAttribute[] UmlAttributes
        {
            get
            {
                return this.umlAttributes.ToArray();
            }
        }

        /// <summary>
        /// Add a reference to an attribute for this class.
        /// </summary>
        /// <param name="xmiIdRef">The id of the attribute to reference.</param>
        public void AddAttributeReference(string xmiIdRef)
        {
            if (!this.attributeReferences.Contains(xmiIdRef))
            {
                this.attributeReferences.Add(xmiIdRef);
            }
        }

        /// <summary>
        /// Gets an array of attribute reference ids.
        /// </summary>
        public string[] AttributeReferences
        {
            get
            {
                return this.attributeReferences.ToArray();
            }
        }

        /// <summary>
        /// Add a method id reference to this class.
        /// </summary>
        /// <param name="xmiIdRef">The id to reference.</param>
        public void AddMethodReference(string xmiIdRef)
        {
            if (!this.methodReferences.Contains(xmiIdRef))
            {
                this.methodReferences.Add(xmiIdRef);
            }
        }

        /// <summary>
        /// Gets an array of methods referenced by this class.
        /// </summary>
        public string[] MethodReferences
        {
            get
            {
                return this.methodReferences.ToArray();
            }
        }

        /// <summary>
        /// Gets a string representing the type of this class, including interface and abstract.
        /// </summary>
        private string ClassType
        {
            get
            {
                if (this.IsInterface)
                {
                    return "interface";
                }
                else if (this.isAbstract)
                {
                    return "abstract class";
                }
                else
                {
                    return "class";
                }
            }
        }

        /// <summary>
        /// Gets the name of this class.
        /// </summary>
        public string Name
        {
            get
            {
                return this.name;
            }
        }

        /// <summary>
        /// Gets the namespace of this class.
        /// </summary>
        public string Namespace
        {
            get
            {
                return this.@namespace;
            }
            set
            {
                this.@namespace = value;
            }
        }

        /// <summary>
        /// Write data about this class to an XmlWriter.
        /// </summary>
        /// <param name="writer">The XmlWriter to write to.</param>
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartDocument();
            writer.WriteStartElement("DATA");
            writer.WriteElementString("visibility", this.visibility);
            writer.WriteElementString("classType", this.ClassType);
            writer.WriteElementString("name", this.Name);
            writer.WriteElementString("namespace", this.Namespace);
            string documentation = Helper.GetDocumentation(this, 8);
            documentation += Helper.FormatText("<author>TCSDESIGNER</author>", 8, 120)[0] + "\r\n";
            documentation += Helper.FormatText("<author>TCSDEVELOPER</author>", 8, 120)[0] + "\r\n";
            documentation += Helper.FormatText("<version>1.0</version>", 8, 120)[0] + "\r\n";
            documentation += Helper.FormatText("<copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>", 8, 120)[0];
            writer.WriteElementString("documentation", documentation);
            foreach (string include in this.Includes)
            {
                writer.WriteStartElement("include");
                writer.WriteElementString("namespace", include);
                writer.WriteEndElement();
            }
            foreach (UmlAttribute umlAttribute in this.umlAttributes)
            {
                umlAttribute.WriteXml(writer);
            }
            foreach (Method method in this.methods)
            {
                method.WriteXml(writer);
            }
            writer.WriteEndElement();
            writer.WriteEndDocument();
        }
    }
}