// Copyright (c) 2006, TopCoder, Inc. All rights reserved.
using System;
using System.Collections.Generic;
using System.Xml;

namespace TopCoder.Uml.Xmi.MetaData
{
    /// <summary>
    /// Represents a method parameter.
    /// </summary>
    /// <author>Dan Manges (Seiz3r)</author>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class UmlParameter : XmiElement
    {
        /// <summary>
        /// Information about the parameter.
        /// </summary>
        private string name, kind, type, @namespace;

        /// <summary>
        /// A TypeRef to store a reference to determine the type of the parameter.
        /// </summary>
        private TypeRef typeRef;

        /// <summary>
        /// Create a new parameter from a XMI node.
        /// </summary>
        /// <param name="node">The XMI node to read data from.</param>
        public UmlParameter(XmlNode node)
            : base(node)
        {
            this.kind = Helper.ReadAttribute(node, "kind");
            this.name = Helper.ReadAttribute(node, "name");
            ReadTypeRef(node);
        }

        /// <summary>
        /// Gets the name of the parameter.
        /// </summary>
        public string Name
        {
            get
            {
                return this.name;
            }
        }

        /// <summary>
        /// Gets or sets the type of the parameter.
        /// </summary>
        public string Type
        {
            get
            {
                return this.type;
            }
            set
            {
                this.type = value;
            }
        }

        /// <summary>
        /// Gets the kind of the parameter.
        /// </summary>
        /// <remarks>Some possible values: in, out, return.</remarks>
        public string Kind
        {
            get
            {
                return this.kind;
            }
        }

        /// <summary>
        /// Gets whether this parameter is a return parameter.
        /// </summary>
        public bool IsReturnParameter
        {
            get
            {
                return this.kind == "return";
            }
        }

        /// <summary>
        /// Gets or sets the namespace for the type of this parameter.
        /// </summary>
        public string Namespace
        {
            get
            {
                return this.@namespace;
            }
            set
            {
                if (!value.Contains("TopCoder") && !value.Contains("System"))
                {
                    this.@namespace = "System." + value;
                }
                else
                {
                    this.@namespace = value;
                }
            }
        }

        /// <summary>
        /// Gets the TypeRef for a reference to the type and id of this parameter type.
        /// </summary>
        public TypeRef TypeRef
        {
            get
            {
                return this.typeRef;
            }
        }

        /// <summary>
        /// Read the type and reference id for the type of this parameter.
        /// </summary>
        /// <param name="node">The node to read from.</param>
        private void ReadTypeRef(XmlNode node)
        {
            foreach (XmlNode childNode in node.ChildNodes)
            {
                if (childNode.Name == "UML:Parameter.type" || childNode.Name == "UML2:TypedElement.type")
                {
                    if (childNode.ChildNodes.Count == 1)
                    {
                        string idRef = Helper.ReadAttribute(childNode.ChildNodes[0], "xmi.idref");
                        string nodeType = childNode.ChildNodes[0].LocalName;
                        UmlType umlType = Helper.StringToUmlType(nodeType);
                        this.typeRef = new TypeRef(umlType, idRef);
                        break;
                    }
                }
            }
        }

        /// <summary>
        /// Write the parameter data to an XmlWriter.
        /// </summary>
        /// <remarks>
        /// Parameter documentation is included in the method documentation.
        /// </remarks>
        /// <param name="writer">The XmlWriter to use for output.</param>
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement("parameter");
            writer.WriteElementString("kind", this.Kind);
            writer.WriteElementString("type", this.Type);
            writer.WriteElementString("name", this.Name);
            writer.WriteEndElement();
        }
    }
}