using System;
using System.IO;
using NUnit.Framework;

namespace TopCoder.Uml.Xmi
{
    /// <summary>
    /// Test class for CodeGenerator.
    /// </summary>
    /// <author>Dan Manges (Seiz3r)</author>
    [TestFixture]
    public class CodeGeneratorTest
    {
        /// <summary>
        /// XMI test file.
        /// </summary>
        private const string XMI_TEST_FILE = @"..\..\test_files\1.4.1\xsd2ddl9.xmi";

        private static string[] XMI_TEST_FILES = new string[] {
            @"..\..\test_files\1.4.1\Complex_Boolean_Expression_Evaluator_1.0.xmi",
            @"..\..\test_files\1.4.3\Audit_Log.xmi",
            @"..\..\test_files\1.4.3\Bloom_Filter_1.0.xmi",
            @"..\..\test_files\1.4.3\DB_Table_Object_Generator.xmi",
            @"..\..\test_files\1.4.3\JavaScript_Drag_and_Drop.xmi",
            @"..\..\test_files\1.4.3\CommandLineExecutor1.0.xmi",
            @"..\..\test_files\1.4.3\Helper_Table_Manager_012.xmi",
            @"..\..\test_files\1.4.3\ObjectDiff.xmi"};

        /// <summary>
        /// Temporary directory to use for testing.
        /// </summary>
        private string tempDirectory;

        /// <summary>
        /// CodeGenerator to use for testing
        /// </summary>
        private CodeGenerator codeGenerator;

        /// <summary>
        /// Initialize variables needed for testing.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            this.tempDirectory = this.GetTempDirectory();
            this.codeGenerator = new CodeGenerator(XMI_TEST_FILE, this.tempDirectory);
            this.codeGenerator.Execute();
        }

        /// <summary>
        /// Remove temporary files and set variables to null.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            try
            {
                Directory.Delete(this.tempDirectory, true);
            }
            catch (IOException)
            {
                Console.WriteLine("Could not delete directory: " + this.tempDirectory);
            }
            this.tempDirectory = null;
            this.codeGenerator = null;
        }

        /// <summary>
        /// Test generating code for several xmi files.
        /// </summary>
        [Test]
        public void TestSeveralXmiFiles()
        {
            foreach (string file in XMI_TEST_FILES)
            {
                this.codeGenerator = new CodeGenerator(file, this.tempDirectory);
                this.codeGenerator.Execute();
                Assert.IsTrue(Directory.GetDirectories(this.tempDirectory).Length > 0);
            }
        }

        /// <summary>
        /// Test ensuring source code files are present, only looking for a few of the files.
        /// </summary>
        [Test]
        public void TestFilesArePresent()
        {
            Assert.IsTrue(File.Exists(this.tempDirectory + @"\" + @"TopCoder\DB\Generation\Xsd2Ddl\DdlGeneratorUtility.cs"));
            Assert.IsTrue(File.Exists(this.tempDirectory + @"\" + @"TopCoder\DB\Generation\Xsd2Ddl\Generators\AbstractDdlGenerator.cs"));
            Assert.IsTrue(File.Exists(this.tempDirectory + @"\" + @"TopCoder\DB\Generation\Xsd2Ddl\Generators\ANSIDatabaseGenerator.cs"));
            Assert.IsTrue(File.Exists(this.tempDirectory + @"\" + @"TopCoder\DB\Generation\Xsd2Ddl\IStandard.cs"));
            Assert.IsTrue(File.Exists(this.tempDirectory + @"\" + @"TopCoder\DB\Generation\Xsd2Ddl\Sources\MicrosoftXsdSource.cs"));
            Assert.IsTrue(File.Exists(this.tempDirectory + @"\" + @"TopCoder\DB\Generation\Xsd2Ddl\Standards\ValidateColumnName.cs"));
        }

        /// <summary>
        /// Ensure each source code file has a copyright and using system.
        /// </summary>
        [Test]
        public void TestCopyrightAndUsingSystem()
        {
            string[] files = Directory.GetFiles(this.tempDirectory, "*.cs");
            foreach (string file in files)
            {
                Assert.IsTrue(this.FileContains(file, "Copyright"));
                Assert.IsTrue(this.FileContains(file, "using System;"));
            }
        }

        /// <summary>
        /// Check some class declarations.
        /// </summary>
        [Test]
        public void TestClassDeclarations()
        {
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\Standards\AbstractStandard.cs", "public abstract class AbstractStandard"));
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\Standards\ValidateColumnName.cs", "public class ValidateColumnName"));
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\IFileSource.cs", "public interface IFileSource"));
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\UnknownFileTypeException.cs", "public class UnknownFileTypeException"));
        }

        /// <summary>
        /// Check for some method declarations.
        /// </summary>
        [Test]
        public void TestMethodDeclarations()
        {
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\DdlGeneratorUtility.cs",
                "private static void ValidateConfigNamespace(CommandLineSwitch switch)"));
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\IDdlGenerator.cs",
                "public string GenerateDdl(DataSet dataSet);"));
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\Xsd2DdlException.cs",
                "protected  Xsd2DdlException(SerializationInfo info, StreamingContext context)"));
        }

        /// <summary>
        /// Check for property declarations.
        /// </summary>
        [Test]
        public void TestPropertyDeclaractions()
        {
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\Standards\AndStandard.cs",
                "public AbstractStandard Left"));
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\Standards\AndStandard.cs",
                "private readonly AbstractStandard left"));
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\Standards\AbstractDataValidationStandard.cs",
                "public IValidator Validator"));
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\Standards\AbstractDataValidationStandard.cs",
                "private readonly IValidator validator"));
            Assert.IsTrue(this.FileContains(this.tempDirectory + @"\" +
                @"TopCoder\DB\Generation\Xsd2Ddl\DdlGeneratorUtility.cs",
                "private readonly MagicNumber magicNumber"));
        }

        /// <summary>
        /// Determines if a file contains a search string.
        /// </summary>
        /// <param name="file">The file to read.</param>
        /// <param name="search">The string to search for.</param>
        /// <returns>True if the file contains the string, false otherwise.</returns>
        private bool FileContains(string file, string search)
        {
            string content = "";
            using (StreamReader reader = new StreamReader(file))
            {
                content = reader.ReadToEnd();
            }
            return content.Contains(search);
        }

        /// <summary>
        /// Gets a temporary directory.
        /// </summary>
        /// <returns>A temporary directory.</returns>
        private string GetTempDirectory()
        {
            string tempFile = System.IO.Path.GetTempFileName();
            File.Delete(tempFile);
            Directory.CreateDirectory(tempFile);
            return tempFile;
        }
    }
}
